#ifndef __MD_LIB_DATETIME_H__
#define __MD_LIB_DATETIME_H__

#include <ostream>

#include <MD/MD_sys_types.h>
#include <MD/private/prefix.h>


namespace MD {

  class MD_LIB_API MD_DateTime
  {
  public:
    /*
    * date - digit format <YYYYMMDD> (Y = year, M = month, D = day).
    * time - digit format <HHMMSSmmm> (H = hours, M = minutes, S = seconds,
    *        m = milliseconds).
    * exch_gmt_offset - Exchange GMT Offset associated with this time (not
    *                   necessarily the GMT Offset this time is currently
    *                   in though). GMT Offset is in minutes.
    */
    MD_DateTime(int date, int time, int16_t exch_gmt_offset);

    int date() const;
    int time() const;
    int16_t exch_gmt_offset() const;



    /*
    * This constructor is for deserializing into only, there are no modifiers.
    */
    MD_DateTime();

    /*
    * Static serialization interface
    */
    static void serialize(char *dst, const MD_DateTime &src);
    static void deserialize(MD_DateTime &dst, const char *src);


  private:
    /*
    * See comments in source code for detailed packing information.
    */
    uint64_t m_packed;

  };

  MD_LIB_API std::ostream &operator<<(std::ostream &os, const MD_DateTime &dt);
}

#endif
