#ifndef __MD_LIB_FILE_UTILS_H_
#define __MD_LIB_FILE_UTILS_H_

#include <string>

#include <MD/MD_sys_types.h>
#include <MD/private/prefix.h>

namespace MD {

  class MD_LIB_API FileSearchParameter;
  MD_LIB_API std::string file_search( const char* target,
    const FileSearchParameter & parameter,
    const char* productName = NULL,
    const char* companyName = "FactSet" );

#ifdef MD_LIB_WIN
  MD_LIB_API std::wstring GetProgramDataFolder();
  MD_LIB_API std::wstring GetModuleFolder();
#endif

  class MD_LIB_API FileSearchParameter {
  public:
    FileSearchParameter(const char* env_variable);
    FileSearchParameter(const std::string & env_variable);
    FileSearchParameter & default_path(const std::string & default_path);

  private:
#ifdef MD_LIB_WIN
#pragma warning(push)
#pragma warning(disable:4251)
#endif
    std::string m_env_variable;
    std::string m_default_path;
#ifdef MD_LIB_WIN
#pragma warning(pop)
#endif

  // pay no attention to anything beyond
  // this stuff allows us to safely construct struct literals pre-main
  // calls to file_search will simply construct a FileSearchParameter from these literals
  public:
    struct literal_t {
      const char* env_variable;
      const char* default_path;
    };
    FileSearchParameter(const literal_t&);

    friend MD_LIB_API std::string file_search( const char* target,
      const FileSearchParameter & parameter,
      const char* productName, const char* companyName);
  };

  extern MD_LIB_API FileSearchParameter::literal_t CONFIG_PATH;

} // namespace

#endif // __MD_LIB_FILE_UTILS_H_
